@@ -2,10 +2,12 @@
 Utilities - Helper functions.
 """
 
+import calendar
 import collections
 import datetime
 import os
 import re
+import socket
 import urllib.parse
 
 import bs4
@@ -17,8 +19,11 @@ import pkg_resources
 import requests
 from pyhelpers.ops import confirmed, fake_requests_headers
 from pyhelpers.store import load_json, load_pickle, save_json, save_pickle
+from pyhelpers.text import find_similar_str
 
 
+# -- Specification of resource homepage ------------------------------------------------
+
 def homepage_url():
     """
     Specify the homepage URL of the data source.
@@ -26,10 +31,11 @@ def homepage_url():
     :return: URL of the data source homepage
     :rtype: str
     """
+
     return 'http://www.railwaycodes.org.uk/'
 
 
-# -- Directory -------------------------------------------------------------------------
+# -- Specification of directory/file paths ---------------------------------------------
 
 def cd_dat(*sub_dir, dat_dir="dat", mkdir=False, **kwargs):
     """
@@ -55,21 +61,25 @@ def cd_dat(*sub_dir, dat_dir="dat", mkdir=False, **kwargs):
         >>> path_to_dat_dir = cd_dat("line-data", dat_dir="dat", mkdir=False)
         >>> print(os.path.relpath(path_to_dat_dir))
         pyrcs\\dat\\line-data
+
+    :meta private:
     """
 
     path = pkg_resources.resource_filename(__name__, dat_dir)
     for x in sub_dir:
         path = os.path.join(path, x)
+
     if mkdir:
         path_to_file, ext = os.path.splitext(path)
         if ext == '':
             os.makedirs(path_to_file, exist_ok=True, **kwargs)
         else:
             os.makedirs(os.path.dirname(path_to_file), exist_ok=True, **kwargs)
+
     return path
 
 
-# -- Converters ------------------------------------------------------------------------
+# -- Data converters -------------------------------------------------------------------
 
 def mile_chain_to_nr_mileage(miles_chains):
     """
@@ -101,6 +111,7 @@ def mile_chain_to_nr_mileage(miles_chains):
         network_rail_mileage = '%.4f' % (int(miles) + round(yards / (10 ** 4), 4))
     else:
         network_rail_mileage = ''
+
     return network_rail_mileage
 
 
@@ -134,6 +145,7 @@ def nr_mileage_to_mile_chain(str_mileage):
         miles_chains = '%.2f' % (int(miles) + round(chains / (10 ** 2), 2))
     else:
         miles_chains = ''
+
     return miles_chains
 
 
@@ -162,6 +174,7 @@ def nr_mileage_str_to_num(str_mileage):
     """
 
     num_mileage = np.nan if str_mileage == '' else round(float(str_mileage), 4)
+
     return num_mileage
 
 
@@ -228,11 +241,11 @@ def nr_mileage_to_yards(nr_mileage):
 
     if isinstance(nr_mileage, (float, np.float, int, np.integer)):
         nr_mileage = nr_mileage_num_to_str(nr_mileage)
-    else:
-        pass
+
     miles = int(nr_mileage.split('.')[0])
     yards = int(nr_mileage.split('.')[1])
     yards += int(measurement.measures.Distance(mi=miles).yd)
+
     return yards
 
 
@@ -241,7 +254,7 @@ def yards_to_nr_mileage(yards):
     Convert yards to Network Rail mileages.
 
     :param yards: yards
-    :type yards: int or float, numpy.nan, None
+    :type yards: int or float or numpy.nan or None
     :return: Network Rail mileage in the form '<miles>.<yards>'
     :rtype: str
 
@@ -278,6 +291,7 @@ def yards_to_nr_mileage(yards):
         mileage = str('%.4f' % round((mileage_mi + mileage_yd / (10 ** 4)), 4))
     else:
         mileage = ''
+
     return mileage
 
 
@@ -312,6 +326,7 @@ def shift_num_nr_mileage(nr_mileage, shift_yards):
     yards = nr_mileage_to_yards(nr_mileage) + shift_yards
     shifted_nr_mileage = yards_to_nr_mileage(yards)
     shifted_num_mileage = nr_mileage_str_to_num(shifted_nr_mileage)
+
     return shifted_num_mileage
 
 
@@ -335,10 +350,11 @@ def year_to_financial_year(date):
     """
 
     financial_date = date + pd.DateOffset(months=-3)
+
     return financial_date.year
 
 
-# -- Parsers ---------------------------------------------------------------------------
+# -- Data parsers ----------------------------------------------------------------------
 
 def parse_tr(header, trs):
     """
@@ -393,7 +409,6 @@ def parse_tr(header, trs):
     row_spanned = []
     for no, tr in enumerate(trs):
         for td_no, rho in enumerate(tr.find_all('td')):
-            # print(data.has_attr("rowspan"))
             if rho.has_attr('rowspan'):
                 row_spanned.append((no, int(rho['rowspan']), td_no, rho.text))
 
@@ -408,7 +423,7 @@ def parse_tr(header, trs):
             for y in to_repeat:
                 for j in range(1, y[0]):
                     if y[2] in tbl_lst[i] and y[2] != '\xa0':
-                        y[1] += np.abs(tbl_lst[i].index(y[2]) - y[1])
+                        y[1] += np.abs(tbl_lst[i].index(y[2]) - y[1], dtype='int64')
                     tbl_lst[i + j].insert(y[1], y[2])
 
     # if row_spanned:
@@ -474,6 +489,7 @@ def parse_table(source, parser='lxml'):
     header = [header.text for header in headers.find_all('th')]
     # Get a list of lists, each of which corresponds to a piece of record
     trs = table_temp[1:]
+
     # Return a list of parsed tr's, each of which corresponds to one df row
     return parse_tr(header, trs), header
 
@@ -546,7 +562,7 @@ def parse_location_name(location_name):
                 r' \(definition unknown\)|'
                 r' \(reopened |'
                 r'( portion])$')
-            x_tmp = re.search(r'(?=[\[(]).*(?<=[\])])|(?=\().*(?<=\) \[)', location_name)
+            x_tmp = re.search(r'(?=[\[(]).*(?<=[])])|(?=\().*(?<=\) \[)', location_name)
             x_tmp = x_tmp.group() if x_tmp is not None else location_name
             if re.search(m_pattern, location_name):
                 dat = ' '.join(location_name.replace(x_tmp, '').split())
@@ -562,7 +578,7 @@ def parse_location_name(location_name):
             if n is None:
                 n = re.search(
                     r'(?<=(\[[\'\"]\()|(\([\'\"]\[)|(\) \[)).*'
-                    r'(?=(\)[\'\"]\])|(\][\'\"]\))|\])', y)
+                    r'(?=(\)[\'\"]])|(][\'\"]\))|])', y)
             elif '"now deleted"' in y and y.startswith('(') and y.endswith(')'):
                 n = re.search(r'(?<=\().*(?=\))', y)
             note = n.group() if n is not None else ''
@@ -582,11 +598,10 @@ def parse_date(str_date, as_date_type=False):
 
     :param str_date: string-type date
     :type str_date: str
-    :param as_date_type: whether to return the date as `datetime.date`_,
-        defaults to ``False``
+    :param as_date_type: whether to return the date as `datetime.date`_, defaults to ``False``
     :type as_date_type: bool
     :return: parsed date as a string or `datetime.date`_
-    :rtype: str, datetime.date
+    :rtype: str or datetime.date
 
     .. _`datetime.date`: https://docs.python.org/3/library/datetime.html#datetime.date
 
@@ -603,18 +618,159 @@ def parse_date(str_date, as_date_type=False):
         <class 'datetime.date'>
     """
 
-    temp_date = dateutil.parser.parse(str_date, fuzzy=True)
-    # or, temp_date = datetime.strptime(last_update_date[12:], '%d %B %Y')
+    try:
+        temp_date = dateutil.parser.parse(str_date, fuzzy=True)
+        # or, temp_date = datetime.datetime.strptime(str_date[12:], '%d %B %Y')
+    except (TypeError, calendar.IllegalMonthError):
+        month_name = find_similar_str(str_date, calendar.month_name)
+        err_month_ = find_similar_str(month_name, str_date.split(' '))
+        temp_date = dateutil.parser.parse(str_date.replace(err_month_, month_name), fuzzy=True)
 
     parsed_date = temp_date.date() if as_date_type else str(temp_date.date())
 
     return parsed_date
 
 
-# -- Get useful information ------------------------------------------------------------
+# -- Retrieval of useful information ---------------------------------------------------
+
+def get_site_map(update=False, confirmation_required=True, verbose=False):
+    """
+    Fetch the `site map <http://www.railwaycodes.org.uk/misc/sitemap.shtm>`_
+    from the package data.
+
+    :param update: whether to check on update and proceed to update the package data,
+        defaults to ``False``
+    :type update: bool
+    :param confirmation_required: whether to prompt a message for confirmation to proceed,
+        defaults to ``True``
+    :type confirmation_required: bool
+    :param verbose: whether to print relevant information in console as the function runs,
+        defaults to ``False``
+    :type verbose: bool or int
+    :return: dictionary of site map data
+    :rtype: dict or None
+
+    **Examples**::
+
+        >>> from pyrcs.utils import get_site_map
+
+        >>> site_map_dat = get_site_map()
+
+        >>> type(site_map_dat)
+        <class 'dict'>
+        >>> print(list(site_map_dat.keys()))
+        ['Home', 'Line data', 'Other assets', '"Legal/financial" lists', 'Miscellaneous']
+        >>> print(site_map_dat['Home'])
+        http://www.railwaycodes.org.uk/index.shtml
+
+        >>> site_map_dat = get_site_map(update=True, verbose=2)
+    """
+
+    path_to_pickle = cd_dat("site-map.pickle", mkdir=True)
+
+    if os.path.isfile(path_to_pickle) and not update:
+        site_map = load_pickle(path_to_pickle)
+
+    else:
+        site_map = None
+
+        if confirmed("To collect the site map?",
+                     confirmation_required=confirmation_required):
+
+            if verbose == 2:
+                print("Updating the package data", end=" ... ")
+
+            url = urllib.parse.urljoin(homepage_url(), '/misc/sitemap.shtm')
+
+            try:
+                source = requests.get(url, headers=fake_requests_headers())
+            except requests.exceptions.ConnectionError:
+                print_conn_err(update=update, verbose=True if update else verbose)
+
+            else:
+                try:
+                    soup = bs4.BeautifulSoup(source.text, 'lxml')
+
+                    # <h3>
+                    h3 = [x.get_text(strip=True) for x in soup.find_all('h3')]
+
+                    site_map = {}
+
+                    # Next <ol>
+                    next_ol = soup.find('h3').find_next('ol')
+
+                    for i in range(len(h3)):
+
+                        li_tag = next_ol.findChildren('li')
+                        ol_tag = next_ol.findChildren('ol')
+
+                        if not ol_tag:
+                            dat_ = [x.find('a').get('href') for x in li_tag]
+                            if len(dat_) == 1:
+                                dat = urllib.parse.urljoin(homepage_url(), dat_[0])
+                            else:
+                                dat = [urllib.parse.urljoin(homepage_url(), x) for x in dat_]
+                            site_map.update({h3[i]: dat})
+
+                        else:
+                            site_map_ = {}
+                            for ol in ol_tag:
+                                k = ol.find_parent('ol').find_previous('li').get_text(
+                                    strip=True)
+
+                                if k not in site_map_.keys():
+                                    sub_li = ol.findChildren('li')
+                                    sub_ol = ol.findChildren('ol')
+
+                                    if sub_ol:
+                                        cat0 = [x.get_text(strip=True) for x in sub_li
+                                                if not x.find('a')]
+                                        dat0 = [[urllib.parse.urljoin(homepage_url(),
+                                                                      a.get('href'))
+                                                 for a in x.find_all('a')] for x in sub_ol]
+                                        cat_name = ol.find_previous('li').get_text(strip=True)
+                                        if cat0:
+                                            site_map_.update(
+                                                {cat_name: dict(zip(cat0, dat0))})
+                                        else:
+                                            site_map_.update(
+                                                {cat_name: [x_ for x in dat0 for x_ in x]})
+                                        # cat_ = [x for x in cat_ if x not in cat0]
+
+                                    else:
+                                        cat_name_ = ol.find_previous('li').get_text(
+                                            strip=True)
+                                        pat = r'.+(?= \(the thousands of mileage files)'
+                                        cat_name = re.search(pat, cat_name_).group(0) \
+                                            if re.match(pat, cat_name_) else cat_name_
+
+                                        dat0 = [urllib.parse.urljoin(homepage_url(),
+                                                                     x.a.get('href'))
+                                                for x in sub_li]
+
+                                        site_map_.update({cat_name: dat0})
+
+                            site_map.update({h3[i]: site_map_})
+
+                        if i < len(h3) - 1:
+                            next_ol = next_ol.find_next('h3').find_next('ol')
+
+                    print("Done. ") if verbose == 2 else ""
+
+                    if site_map is not None:
+                        save_pickle(site_map, path_to_pickle, verbose=verbose)
+
+                except Exception as e:
+                    print("Failed. {}".format(e))
+
+        else:
+            print("Cancelled. ") if verbose == 2 else ""
+            site_map = load_pickle(path_to_pickle)
+
+    return site_map
 
 
-def get_last_updated_date(url, parsed=True, as_date_type=False):
+def get_last_updated_date(url, parsed=True, as_date_type=False, verbose=False):
     """
     Get last update date.
 
@@ -625,8 +781,11 @@ def get_last_updated_date(url, parsed=True, as_date_type=False):
     :param as_date_type: whether to return the date as `datetime.date`_,
         defaults to ``False``
     :type as_date_type: bool
+    :param verbose: whether to print relevant information in console as the function runs,
+        defaults to ``False``
+    :type verbose: bool or int
     :return: date of when the specified web page was last updated
-    :rtype: str, datetime.date, None
+    :rtype: str or datetime.date or None
 
     .. _`datetime.date`: https://docs.python.org/3/library/datetime.html#datetime.date
 
@@ -634,43 +793,50 @@ def get_last_updated_date(url, parsed=True, as_date_type=False):
 
         >>> from pyrcs.utils import get_last_updated_date
 
-        >>> last_update_date_ = get_last_updated_date(
+        >>> last_upd_date = get_last_updated_date(
         ...     url='http://www.railwaycodes.org.uk/crs/CRSa.shtm', parsed=True,
         ...     as_date_type=False)
-        >>> type(last_update_date_)
+        >>> type(last_upd_date)
         <class 'str'>
 
-        >>> last_update_date_ = get_last_updated_date(
+        >>> last_upd_date = get_last_updated_date(
         ...     url='http://www.railwaycodes.org.uk/crs/CRSa.shtm', parsed=True,
         ...     as_date_type=True)
-        >>> type(last_update_date_)
+        >>> type(last_upd_date)
         <class 'datetime.date'>
 
-        >>> last_update_date_ = get_last_updated_date(
+        >>> last_upd_date = get_last_updated_date(
         ...     url='http://www.railwaycodes.org.uk/linedatamenu.shtm')
-        >>> print(last_update_date_)
+        >>> print(last_upd_date)
         None
     """
 
+    last_update_date = None
+
     # Request to get connected to the given url
-    source = requests.get(url, headers=fake_requests_headers())
-    web_page_text = source.text
-
-    # Parse the text scraped from the requested web page
-    parsed_text = bs4.BeautifulSoup(web_page_text, 'lxml')
-    # Find 'Last update date'
-    update_tag = parsed_text.find('p', {'class': 'update'})
-
-    if update_tag is not None:
-        last_update_date = update_tag.text
-
-        # Decide whether to convert the date's format
-        if parsed:
-            # Convert the date to "yyyy-mm-dd" format
-            last_update_date = parse_date(last_update_date, as_date_type)
+    try:
+        source = requests.get(url, headers=fake_requests_headers())
+    except requests.exceptions.ConnectionError:
+        print_connection_error(verbose=verbose)
 
     else:
-        last_update_date = None  # print('Information not available.')
+        web_page_text = source.text
+
+        # Parse the text scraped from the requested web page
+        parsed_text = bs4.BeautifulSoup(web_page_text, 'lxml')
+        # Find 'Last update date'
+        update_tag = parsed_text.find('p', {'class': 'update'})
+
+        if update_tag is not None:
+            last_update_date = update_tag.text
+
+            # Decide whether to convert the date's format
+            if parsed:
+                # Convert the date to "yyyy-mm-dd" format
+                last_update_date = parse_date(last_update_date, as_date_type)
+
+        # else:
+        #     last_update_date = None  # print('Information not available.')
 
     return last_update_date
 
@@ -692,9 +858,9 @@ def get_catalogue(page_url, update=False, confirmation_required=True, json_it=Tr
     :type json_it: bool
     :param verbose: whether to print relevant information in console as the function runs,
         defaults to ``False``
-    :type verbose: bool
+    :type verbose: bool or int
     :return: catalogue in the form {'<title>': '<URL>'}
-    :rtype: dict
+    :rtype: dict or None
 
     **Examples**::
 
@@ -722,36 +888,47 @@ def get_catalogue(page_url, update=False, confirmation_required=True, json_it=Tr
     path_to_cat_json = cd_dat("catalogue", cat_json, mkdir=True)
 
     if os.path.isfile(path_to_cat_json) and not update:
-        catalogue = load_json(path_to_cat_json, verbose=verbose)
+        catalogue = load_json(path_to_cat_json)
 
     else:
-        if confirmed("To collect/update catalogue? ",
+        catalogue = None
+
+        if confirmed("To collect/update catalogue?",
                      confirmation_required=confirmation_required):
 
-            source = requests.get(page_url, headers=fake_requests_headers())
-            source_text = source.text
-            source.close()
-
             try:
-                cold_soup = \
-                    bs4.BeautifulSoup(source_text, 'lxml').find(
-                        'div', attrs={'class': 'fixed'})
-                catalogue = {
-                    a.get_text(strip=True): urllib.parse.urljoin(page_url, a.get('href'))
-                    for a in cold_soup.find_all('a')}
-            except AttributeError:
-                cold_soup = \
-                    bs4.BeautifulSoup(source_text, 'lxml').find('h1').find_all_next('a')
-                catalogue = {
-                    a.get_text(strip=True): urllib.parse.urljoin(page_url, a.get('href'))
-                    for a in cold_soup}
+                source = requests.get(page_url, headers=fake_requests_headers())
+            except requests.exceptions.ConnectionError:
+                print_connection_error(verbose=verbose)
 
-            if json_it:
-                save_json(catalogue, path_to_cat_json, verbose=verbose)
+            else:
+                try:
+                    source_text = source.text
+                    source.close()
+
+                    try:
+                        cold_soup = bs4.BeautifulSoup(
+                            source_text, 'lxml').find('div', attrs={'class': 'fixed'})
+                        catalogue = {
+                            a.get_text(strip=True):
+                                urllib.parse.urljoin(page_url, a.get('href'))
+                            for a in cold_soup.find_all('a')}
+                    except AttributeError:
+                        cold_soup = bs4.BeautifulSoup(
+                            source_text, 'lxml').find('h1').find_all_next('a')
+                        catalogue = {
+                            a.get_text(strip=True):
+                                urllib.parse.urljoin(page_url, a.get('href'))
+                            for a in cold_soup}
+
+                    if json_it and catalogue is not None:
+                        save_json(catalogue, path_to_cat_json, verbose=verbose)
+
+                except Exception as e:
+                    print("Failed to get the category menu. {}".format(e))
 
         else:
             print("The catalogue for the requested data has not been acquired.")
-            catalogue = None
 
     return catalogue
 
@@ -773,9 +950,9 @@ def get_category_menu(menu_url, update=False, confirmation_required=True, json_i
     :type json_it: bool
     :param verbose: whether to print relevant information in console as the function runs,
         defaults to ``False``
-    :type verbose: bool
-    :return:
-    :rtype: dict
+    :type verbose: bool or int
+    :return: a category menu
+    :rtype: dict or None
 
     **Example**::
 
@@ -795,156 +972,81 @@ def get_category_menu(menu_url, update=False, confirmation_required=True, json_i
     path_to_menu_json = cd_dat("catalogue", menu_json, mkdir=True)
 
     if os.path.isfile(path_to_menu_json) and not update:
-        cls_menu = load_json(path_to_menu_json, verbose=verbose)
+        cls_menu = load_json(path_to_menu_json)
 
     else:
-        if confirmed("To collect/update category menu? ",
+        cls_menu = None
+
+        if confirmed("To collect/update category menu?",
                      confirmation_required=confirmation_required):
 
-            source = requests.get(menu_url, headers=fake_requests_headers())
-
-            soup = bs4.BeautifulSoup(source.text, 'lxml')
-            h1, h2s = soup.find('h1'), soup.find_all('h2')
-
-            cls_name = h1.text.replace(' menu', '')
-
-            if len(h2s) == 0:
-                cls_elem = dict((x.text, urllib.parse.urljoin(menu_url, x.get('href')))
-                                for x in h1.find_all_next('a'))
+            try:
+                source = requests.get(menu_url, headers=fake_requests_headers())
+            except requests.exceptions.ConnectionError:
+                print_connection_error(verbose=verbose)
 
             else:
-                all_next = [x.replace(':', '') for x in h1.find_all_next(string=True)
-                            if x != '\n' and x != '\xa0'][2:]
-                h2s_list = [x.text.replace(':', '') for x in h2s]
-                all_next_a = [(x.text, urllib.parse.urljoin(menu_url, x.get('href')))
-                              for x in h1.find_all_next('a', href=True)]
+                try:
+                    soup = bs4.BeautifulSoup(source.text, 'lxml')
+                    h1, h2s = soup.find('h1'), soup.find_all('h2')
 
-                idx = [all_next.index(x) for x in h2s_list]
-                for i in idx:
-                    all_next_a.insert(i, all_next[i])
+                    cls_name = h1.text.replace(' menu', '')
+
+                    if len(h2s) == 0:
+                        cls_elem = dict(
+                            (x.text, urllib.parse.urljoin(menu_url, x.get('href')))
+                            for x in h1.find_all_next('a'))
 
-                cls_elem, i = {}, 0
-                while i <= len(idx):
-                    if i == 0:
-                        d = dict(all_next_a[i:idx[i]])
-                    elif i < len(idx):
-                        d = {h2s_list[i - 1]: dict(all_next_a[idx[i - 1] + 1:idx[i]])}
                     else:
-                        d = {h2s_list[i - 1]: dict(all_next_a[idx[i - 1] + 1:])}
-                    i += 1
-                    cls_elem.update(d)
+                        all_next = [x.replace(':', '')
+                                    for x in h1.find_all_next(string=True)
+                                    if x != '\n' and x != '\xa0'][2:]
+                        h2s_list = [x.text.replace(':', '') for x in h2s]
+                        all_next_a = [
+                            (x.text, urllib.parse.urljoin(menu_url, x.get('href')))
+                            for x in h1.find_all_next('a', href=True)]
 
-            cls_menu = {cls_name: cls_elem}
+                        idx = [all_next.index(x) for x in h2s_list]
+                        for i in idx:
+                            all_next_a.insert(i, all_next[i])
 
-            if json_it:
-                save_json(cls_menu, path_to_menu_json, verbose=verbose)
+                        cls_elem, i = {}, 0
+                        while i <= len(idx):
+                            if i == 0:
+                                d = dict(all_next_a[i:idx[i]])
+                            elif i < len(idx):
+                                d = {h2s_list[i - 1]: dict(
+                                    all_next_a[idx[i - 1] + 1:idx[i]])}
+                            else:
+                                d = {h2s_list[i - 1]: dict(
+                                    all_next_a[idx[i - 1] + 1:])}
+                            i += 1
+                            cls_elem.update(d)
+
+                    cls_menu = {cls_name: cls_elem}
+
+                    if json_it and cls_menu is not None:
+                        save_json(cls_menu, path_to_menu_json, verbose=verbose)
+
+                except Exception as e:
+                    print("Failed to get the category menu. {}".format(e))
 
         else:
             print("The category menu has not been acquired.")
-            cls_menu = None
 
     return cls_menu
 
 
-def get_station_data_catalogue(source_url, source_key, update=False):
-    """
-    Get catalogue of railway station data.
-
-    :param source_url: URL to the source web page
-    :type source_url: str
-    :param source_key: key of the returned catalogue (which is a dictionary)
-    :type source_key: str
-    :param update: whether to check on update and proceed to update the package data,
-        defaults to ``False``
-    :type update: bool
-    :return: catalogue of railway station data
-    :rtype: dict
-
-    See :py:class:`pyrcs.other_assets.Stations()
-    <pyrcs.other_assets.stations.Stations>`
-    """
-
-    cat_json = '-'.join(x for x in urllib.parse.urlparse(source_url).path.replace(
-        '.shtm', '.json').split('/') if x)
-    path_to_cat = cd_dat("catalogue", cat_json)
-
-    if os.path.isfile(path_to_cat) and not update:
-        catalogue = load_json(path_to_cat)
-
-    else:
-        source = requests.get(source_url, headers=fake_requests_headers())
-        cold_soup = bs4.BeautifulSoup(source.text, 'lxml').find(
-            'p', {'class': 'appeal'}).find_next('p').find_next('p')
-        hot_soup = {a.text: urllib.parse.urljoin(source_url, a.get('href'))
-                    for a in cold_soup.find_all('a')}
-
-        catalogue = {source_key: None}
-        for k, v in hot_soup.items():
-            sub_cat = get_catalogue(v, update=True, confirmation_required=False,
-                                    json_it=False)
-            if sub_cat != hot_soup:
-                if k == 'Introduction':
-                    catalogue.update({source_key: {k: v, **sub_cat}})
-                else:
-                    catalogue.update({k: sub_cat})
-            else:
-                if k in ('Bilingual names', 'Not served by SFO'):
-                    catalogue[source_key].update({k: v})
-                else:
-                    catalogue.update({k: v})
-
-        save_json(catalogue, path_to_cat)
-
-    return catalogue
-
-
-def get_track_diagrams_items(source_url, source_key, update=False):
-    """
-    Get catalogue of track diagrams.
-
-    :param source_url: URL to the source web page
-    :type source_url: str
-    :param source_key: key of the returned catalogue (which is a dictionary)
-    :type source_key: str
-    :param update: whether to check on update and proceed to update the package data,
-        defaults to ``False``
-    :type update: bool
-    :return: catalogue of railway station data
-    :rtype: dict
-
-    See :py:class:`pyrcs.line_data.TrackDiagrams()
-    <pyrcs.line_data.track_diagrams.TrackDiagrams>`
-    """
-
-    cat_json = '-'.join(x for x in urllib.parse.urlparse(source_url).path.replace(
-        '.shtm', '.json').split('/') if x)
-    path_to_cat = cd_dat("catalogue", cat_json)
-
-    if os.path.isfile(path_to_cat) and not update:
-        items = load_pickle(path_to_cat)
-
-    else:
-        source = requests.get(source_url, headers=fake_requests_headers())
-        soup = bs4.BeautifulSoup(source.text, 'lxml')
-        h3 = {x.get_text(strip=True)
-              for x in soup.find_all('h3', text=True, attrs={'class': None})}
-        items = {source_key: h3}
-
-        save_pickle(items, path_to_cat)
-
-    return items
-
-
 # -- Rectification of location names ---------------------------------------------------
 
-def fetch_location_names_repl_dict(k=None, regex=False, as_dataframe=False):
+def fetch_loc_names_repl_dict(k=None, regex=False, as_dataframe=False):
     """
     Create a dictionary for rectifying location names.
 
     :param k: key of the created dictionary, defaults to ``None``
     :type k: str or int or float or bool or None
-    :param regex: whether to create a dictionary for replacement
-        based on regular expressions, defaults to ``False``
+    :param regex: whether to create a dictionary for replacement based on regular expressions,
+        defaults to ``False``
     :type regex: bool
     :param as_dataframe: whether to return the created dictionary as a pandas.DataFrame,
         defaults to ``False``
@@ -954,9 +1056,9 @@ def fetch_location_names_repl_dict(k=None, regex=False, as_dataframe=False):
 
     **Examples**::
 
-        >>> from pyrcs.utils import fetch_location_names_repl_dict
+        >>> from pyrcs.utils import fetch_loc_names_repl_dict
 
-        >>> repl_dict = fetch_location_names_repl_dict()
+        >>> repl_dict = fetch_loc_names_repl_dict()
         >>> type(repl_dict)
         <class 'dict'>
         >>> print(list(repl_dict.keys())[:5])
@@ -966,7 +1068,7 @@ def fetch_location_names_repl_dict(k=None, regex=False, as_dataframe=False):
          'Aberdeen Craiginches',
          'Aberdeen Craiginches T.C.']
 
-        >>> repl_dict = fetch_location_names_repl_dict(regex=True, as_dataframe=True)
+        >>> repl_dict = fetch_loc_names_repl_dict(regex=True, as_dataframe=True)
         >>> type(repl_dict)
         <class 'pandas.core.frame.DataFrame'>
         >>> print(repl_dict.head())
@@ -994,9 +1096,9 @@ def fetch_location_names_repl_dict(k=None, regex=False, as_dataframe=False):
     return replacement_dict
 
 
-def update_location_name_repl_dict(new_items, regex, verbose=False):
+def update_loc_names_repl_dict(new_items, regex, verbose=False):
     """
-    Update the location-name replacement dictionary in the package data.
+    Update the location-names replacement dictionary in the package data.
 
     :param new_items: new items to replace
     :type new_items: dict
@@ -1004,35 +1106,33 @@ def update_location_name_repl_dict(new_items, regex, verbose=False):
     :type regex: bool
     :param verbose: whether to print relevant information in console as the function runs,
         defaults to ``False``
-    :type verbose: bool
+    :type verbose: bool or int
 
     **Example**:
 
-        >>> from pyrcs.utils import update_location_name_repl_dict
+        >>> from pyrcs.utils import update_loc_names_repl_dict
 
         >>> new_items_ = {}
-        >>> update_location_name_repl_dict(new_items_, regex=False)
+        >>> update_loc_names_repl_dict(new_items_, regex=False)
     """
 
     json_filename = "location-names-repl{}.json".format("" if not regex else "-regex")
 
     new_items_keys = list(new_items.keys())
 
-    if confirmed("To update \"{}\" with {{\"{}\"... }}?".format(
-            json_filename, new_items_keys[0])):
+    if confirmed("To update \"{}\" with {{\"{}\"... }}?".format(json_filename, new_items_keys[0])):
         path_to_json = cd_dat(json_filename)
         location_name_repl_dict = load_json(path_to_json)
 
         if any(isinstance(k, re.Pattern) for k in new_items_keys):
-            new_items = {k.pattern: v for k, v in new_items.items()
-                         if isinstance(k, re.Pattern)}
+            new_items = {k.pattern: v for k, v in new_items.items() if isinstance(k, re.Pattern)}
 
         location_name_repl_dict.update(new_items)
 
         save_json(location_name_repl_dict, path_to_json, verbose=verbose)
 
 
-# -- Fixers -------------------------------------------------------------------------------------------
+# -- Data fixers -----------------------------------------------------------------------
 
 def fix_num_stanox(stanox_code):
     """
@@ -1111,7 +1211,41 @@ def fix_nr_mileage_str(nr_mileage):
     return nr_mileage_
 
 
-# -- Misc ---------------------------------------------------------------------------------------------
+# -- Miscellaneous helpers -------------------------------------------------------------
+
+def print_connection_error(verbose=False):
+    """
+    Print a message about unsuccessful attempts to establish a connection to the Internet.
+
+    :param verbose: whether to print relevant information in console as the function runs,
+        defaults to ``False``
+    :type verbose: bool or int
+    """
+
+    if verbose:
+        print("Failed to establish an Internet connection. "
+              "The current instance relies on local backup.")
+
+
+def print_conn_err(update=False, verbose=False):
+    """
+    Print a message about unsuccessful attempts to establish a connection to the Internet
+    for an instance of a class.
+
+    :param update: defaults to ``False``
+        (mostly complies with ``update`` in a parent function that uses this function)
+    :type update: bool
+    :param verbose: whether to print relevant information in console as the function runs,
+        defaults to ``False``
+    :type verbose: bool or int
+    """
+
+    msg = "The Internet connection is not available."
+    if update and verbose:
+        print(msg + " Failed to update the data.")
+    elif verbose:
+        print(msg)
+
 
 def is_str_float(str_val):
     """
@@ -1144,4 +1278,29 @@ def is_str_float(str_val):
         test_res = True
     except ValueError:
         test_res = False
+
     return test_res
+
+
+def is_internet_connected():
+    """
+    Check the Internet connection.
+
+    :return: whether the machine is currently connected to the Internet
+    :rtype: bool
+
+    **Examples**::
+
+        >>> from pyrcs.utils import is_internet_connected
+
+        >>> is_internet_connected()
+    """
+
+    try:
+        netloc = urllib.parse.urlparse(homepage_url()).netloc
+        host = socket.gethostbyname(netloc)
+        s = socket.create_connection((host, 80))
+        s.close()
+        return True
+    except (socket.gaierror, OSError):
+        return False
